@@ -61,7 +61,7 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
     while (exfat_find_used_sectors(&ef, &a, &b) == 0){
 	printf("block %" PRId64 " %" PRId64 " \n", a, b);
 	for (block = a; block <= b; block++){
-	    pc_set_bit((uint64_t)block, bitmap);
+	    pc_set_bit((uint64_t)block, bitmap, image_hdr.totalblock);
 	    log_mesg(3, 0, 0, fs_opt.debug, "%s: used block %" PRId64 " \n", __FILE__, block);
 	    /// update progress
 	    update_pui(&prog, block, block, 0);
